// Auth store to manage authentication state
import { defineStore } from 'pinia'

export const useAuthStore = defineStore('auth', {
  state: () => ({
    token: localStorage.getItem('auth_token') || null,
    user: null as any | null,
  }),
  
  getters: {
    isAuthenticated: (state) => !!state.token,
  },
  
  actions: {
    setToken(token: string) {
      this.token = token;
      localStorage.setItem('auth_token', token);
    },
    
    setUser(user: any) {
      this.user = user;
    },
    
    logout() {
      this.token = null;
      this.user = null;
      localStorage.removeItem('auth_token');
    },
    
    async fetchUserInfo() {
      if (!this.token) return;
      
      try {
        const response = await fetch('/api/v1/health', {
          headers: {
            'Authorization': `Bearer ${this.token}`
          }
        });
        
        if (response.ok) {
          const data = await response.json();
          this.setUser(data.user);
        } else {
          // If token is invalid, logout
          this.logout();
        }
      } catch (error) {
        console.error('Failed to fetch user info:', error);
        this.logout();
      }
    }
  }
})
